@@ -1,22 +1,8 @@
-#
 # Copyright (C) 2006, 2013 Red Hat, Inc.
 # Copyright (C) 2006 Hugh O. Brock <hbrock@redhat.com>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
-#
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 import threading
 import traceback
@@ -31,67 +17,65 @@ import virtinst.progress
 from .baseclass import vmmGObjectUI
 
 
-class vmmMeter(virtinst.progress.BaseMeter):
-    def __init__(self, cb_pulse, cb_fraction, cb_done):
-        virtinst.progress.BaseMeter.__init__(self)
-        self.started = False
+class _vmmMeter(virtinst.progress.Meter):
+    def __init__(self, pbar_pulse, pbar_fraction, pbar_done):
+        virtinst.progress.Meter.__init__(self, quiet=True)
 
-        self._vmm_pulse = cb_pulse
-        self._vmm_fraction = cb_fraction
-        self._vmm_done = cb_done
+        self._pbar_pulse = pbar_pulse
+        self._pbar_fraction = pbar_fraction
+        self._pbar_done = pbar_done
 
 
-    def _do_start(self, now=None):
-        if self.text is not None:
-            text = self.text
-        else:
-            text = self.basename
-        if self.size is None:
-            out = "    %5sB" % (0)
-            self._vmm_pulse(out, text)
-        else:
-            out = "%3i%% %5sB" % (0, 0)
-            self._vmm_fraction(0, out, text)
-        self.started = True
+    #################
+    # Internal APIs #
+    #################
 
-    def _do_update(self, amount_read, now=None):
-        if self.text is not None:
-            text = self.text
+    def _write(self):
+        if self._size is None:
+            self._pbar_pulse("", self._text)
         else:
-            text = self.basename
-        fread = virtinst.progress.format_number(amount_read)
-        if self.size is None:
-            out = "    %5sB" % (fread)
-            self._vmm_pulse(out, text)
-        else:
-            frac = self.re.fraction_read()
-            out = "%3i%% %5sB" % (frac * 100, fread)
-            self._vmm_fraction(frac, out, text)
+            fread = virtinst.progress.Meter.format_number(self._total_read)
+            rtime = virtinst.progress.Meter.format_time(
+                    self._meter.re.remaining_time(), True)
+            frac = self._meter.re.fraction_read()
+            out = "%3i%% %5sB %s ETA" % (frac * 100, fread, rtime)
+            self._pbar_fraction(frac, out, self._text)
 
-    def _do_end(self, amount_read, now=None):
-        if self.text is not None:
-            text = self.text
-        else:
-            text = self.basename
-        fread = virtinst.progress.format_number(amount_read)
-        if self.size is None:
-            out = "    %5sB" % (fread)
-            self._vmm_pulse(out, text)
-        else:
-            out = "%3i%% %5sB" % (100, fread)
-            self._vmm_done(out, text)
-        self.started = False
+
+    #############################################
+    # Public APIs specific to virt-manager code #
+    #############################################
+
+    def is_started(self):
+        return bool(self._meter.start_time)
+
+
+    ###################
+    # Meter overrides #
+    ###################
+
+    def start(self, *args, **kwargs):
+        super().start(*args, **kwargs)
+        self._write()
+
+    def update(self, *args, **kwargs):
+        super().update(*args, **kwargs)
+        self._write()
+
+    def end(self, *args, **kwargs):
+        super().end(*args, **kwargs)
+        self._pbar_done()
 
 
 def cb_wrapper(callback, asyncjob, *args, **kwargs):
     try:
         callback(asyncjob, *args, **kwargs)
-    except Exception, e:
+    except Exception as e:
         # If job is cancelled, don't report error to user.
         if (isinstance(e, libvirt.libvirtError) and
             asyncjob.can_cancel() and
             asyncjob.job_canceled):
-            return
+            return  # pragma: no cover
 
         asyncjob.set_error(str(e), "".join(traceback.format_exc()))
 
@@ -182,6 +166,9 @@ class vmmAsyncJob(vmmGObjectUI):
         self._error_info = None
         self._data = None
 
+        self._details_widget = None
+        self._details_update_cb = None
+
         self._is_pulsing = True
         self._meter = None
 
@@ -190,8 +177,7 @@ class vmmAsyncJob(vmmGObjectUI):
         self._bg_thread.daemon = True
 
         self.builder.connect_signals({
-            "on_async_job_delete_event" : self._on_window_delete,
-            "on_async_job_cancel_clicked" : self._on_cancel,
+            "on_async_job_cancel_clicked": self._on_cancel,
         })
 
         # UI state
@@ -215,27 +201,22 @@ class vmmAsyncJob(vmmGObjectUI):
         # pbar idle callbacks and cancel routine which is invoked from the
         # main thread
         if self.job_canceled and not canceling:
-            return
+            return  # pragma: no cover
         self.widget("pbar-stage").set_text(text)
 
-    def _hide_warning(self):
-        self.widget("warning-box").hide()
 
 
     ################
     # UI listeners #
     ################
 
-    def _on_window_delete(self, ignore1=None, ignore2=None):
-        return 1
-
     def _on_cancel(self, ignore1=None, ignore2=None):
         if not self.cancel_cb or not self._bg_thread.is_alive():
-            return
+            return  # pragma: no cover
 
         self.cancel_cb(*self.cancel_args)
-        if self.job_canceled:
-            self._hide_warning()
+        if self.job_canceled:  # pragma: no cover
+            self.widget("warning-box").hide()
             self._set_stage_text(_("Cancelling job..."), canceling=True)
 
 
@@ -245,18 +226,16 @@ class vmmAsyncJob(vmmGObjectUI):
 
     def get_meter(self):
         if not self._meter:
-            self._meter = vmmMeter(self._pbar_pulse,
-                                   self._pbar_fraction,
-                                   self._pbar_done)
+            self._meter = _vmmMeter(self._pbar_pulse,
+                                    self._pbar_fraction,
+                                    self._pbar_done)
         return self._meter
 
     def set_error(self, error, details):
         self._error_info = (error, details)
 
-    def set_extra_data(self, data):
-        self._data = data
-    def get_extra_data(self):
-        return self._data
+    def has_error(self):
+        return bool(self._error_info)
 
     def can_cancel(self):
         return bool(self.cancel_cb)
@@ -286,8 +265,9 @@ class vmmAsyncJob(vmmGObjectUI):
             self.topwin.present()
 
         if not self.cancel_cb and self.show_progress:
-            self.topwin.get_window().set_cursor(
-                            Gdk.Cursor.new(Gdk.CursorType.WATCH))
+            gdk_window = self.topwin.get_window()
+            gdk_window.set_cursor(
+                Gdk.Cursor.new_from_name(gdk_window.get_display(), "progress"))
         self._bg_thread.start()
 
 
@@ -311,14 +291,14 @@ class vmmAsyncJob(vmmGObjectUI):
     @idle_wrapper
     def _pbar_do_pulse(self):
         if not self.builder:
-            return
+            return  # pragma: no cover
         self.widget("pbar").pulse()
 
     @idle_wrapper
     def _pbar_pulse(self, progress="", stage=None):
         self._is_pulsing = True
         if not self.builder:
-            return
+            return  # pragma: no cover
         self.widget("pbar").set_text(progress)
         self._set_stage_text(stage or _("Processing..."))
 
@@ -326,21 +306,26 @@ class vmmAsyncJob(vmmGObjectUI):
     def _pbar_fraction(self, frac, progress, stage=None):
         self._is_pulsing = False
         if not self.builder:
-            return
+            return  # pragma: no cover
         self._set_stage_text(stage or _("Processing..."))
         self.widget("pbar").set_text(progress)
 
-        if frac > 1:
-            frac = 1.0
-        if frac < 0:
-            frac = 0
+        frac = min(frac, 1)
+        frac = max(frac, 0)
         self.widget("pbar").set_fraction(frac)
 
     @idle_wrapper
-    def _pbar_done(self, progress, stage=None):
+    def _pbar_done(self):
         self._is_pulsing = False
-        if not self.builder:
-            return
-        self._set_stage_text(stage or _("Completed"))
-        self.widget("pbar").set_text(progress)
-        self.widget("pbar").set_fraction(1)
+
+    @idle_wrapper
+    def details_enable(self):
+        from gi.repository import Vte
+        self._details_widget = Vte.Terminal()
+        self.widget("details-box").add(self._details_widget)
+        self._details_widget.set_visible(True)
+        self.widget("details").set_visible(True)
+
+    @idle_wrapper
+    def details_update(self, data):
+        self._details_widget.feed(data.replace("\n", "\r\n").encode())
