@@ -104,8 +104,9 @@ class VirtinstConnection(object):
 
     def _get_caps(self):
         if not self._caps:
-            self._caps = Capabilities(self,
-                self._libvirtconn.getCapabilities())
+            capsxml = self._libvirtconn.getCapabilities()
+            self._caps = Capabilities(self, capsxml)
+            log.debug("Fetched capabilities for %s: %s", self._uri, capsxml)
         return self._caps
     caps = property(_get_caps)
 
@@ -113,6 +114,23 @@ class VirtinstConnection(object):
         return self._libvirtconn
 
 
+    ###################
+    # Private helpers #
+    ###################
+
+    def _log_versions(self):
+        def format_version(num):
+            major = int(num / 1000000)
+            minor = int(num / 1000) % 1000
+            micro = num % 1000
+            return "%s.%s.%s" % (major, minor, micro)
+
+        log.debug("libvirt URI versions library=%s driver=%s hypervisor=%s",
+                  format_version(self.local_libvirt_version()),
+                  format_version(self.daemon_version()),
+                  format_version(self.conn_version()))
+
+
     ##############
     # Public API #
     ##############
@@ -162,6 +180,15 @@ class VirtinstConnection(object):
             self._uri = self._libvirtconn.getURI()
             self._uriobj = URI(self._uri)
 
+        self._log_versions()
+        self._get_caps()  # cache and log capabilities
+
+    def get_libvirt_data_root_dir(self):
+        if self.is_privileged():
+            return "/var/lib/libvirt"
+        return os.environ.get("XDG_DATA_HOME",
+                              os.path.expanduser("~/.local/share/libvirt"))
+
 
     ####################
     # Polling routines #
@@ -182,8 +209,16 @@ class VirtinstConnection(object):
     def _fetch_all_domains_raw(self):
         dummy1, dummy2, ret = pollhelpers.fetch_vms(
             self, {}, lambda obj, ignore: obj)
-        return [Guest(weakref.proxy(self), parsexml=obj.XMLDesc(0))
-                for obj in ret]
+        domains = []
+        for obj in ret:
+            # TOCTOU race: a domain may go away in between enumeration and inspection
+            try:
+                xml = obj.XMLDesc(0)
+            except libvirt.libvirtError as e:  # pragma: no cover
+                log.debug("Fetching domain XML failed: %s", e)
+                continue
+            domains.append(Guest(weakref.proxy(self), parsexml=xml))
+        return domains
 
     def _build_pool_raw(self, poolobj):
         return StoragePool(weakref.proxy(self),
@@ -214,7 +249,7 @@ class VirtinstConnection(object):
         # TOCTOU race: a volume may go away in between enumeration and inspection
         try:
             pool = self._libvirtconn.storagePoolLookupByName(poolxmlobj.name)
-        except libvirt.libvirtError as e:  # pragma: no cover
+        except libvirt.libvirtError:  # pragma: no cover
             return ret
 
         if pool.info()[0] != libvirt.VIR_STORAGE_POOL_RUNNING:
@@ -400,6 +435,8 @@ class VirtinstConnection(object):
     def is_vz(self):
         return (self._uriobj.scheme.startswith("vz") or
                 self._uriobj.scheme.startswith("parallels"))
+    def is_bhyve(self):
+        return self._uriobj.scheme.startswith("bhyve")
 
 
     #########################
