@@ -93,6 +93,7 @@ def _make_capsblock(xml_root_name):
 class _SEV(XMLBuilder):
     XML_NAME = "sev"
     supported = XMLProperty("./@supported", is_yesno=True)
+    maxESGuests = XMLProperty("./maxESGuests")
 
 
 #############################
@@ -241,6 +242,8 @@ class DomainCapabilities(XMLBuilder):
             try:
                 xml = conn.getDomainCapabilities(emulator, arch,
                     machine, hvtype)
+                log.debug("Fetched domain capabilities for (%s,%s,%s,%s): %s",
+                          emulator, arch, machine, hvtype, xml)
             except Exception:  # pragma: no cover
                 log.debug("Error fetching domcapabilities XML",
                     exc_info=True)
@@ -390,12 +393,15 @@ class DomainCapabilities(XMLBuilder):
     # Misc support methods #
     ########################
 
-    def supports_sev_launch_security(self):
+    def supports_sev_launch_security(self, check_es=False):
         """
         Returns False if either libvirt doesn't advertise support for SEV at
         all (< libvirt-4.5.0) or if it explicitly advertises it as unsupported
         on the platform
         """
+        if check_es:
+            return bool(self.features.sev.supported and
+                        self.features.sev.maxESGuests)
         return bool(self.features.sev.supported)
 
     def supports_video_bochs(self):
@@ -422,6 +428,14 @@ class DomainCapabilities(XMLBuilder):
         """
         models = self.devices.tpm.get_enum("model").get_values()
         backends = self.devices.tpm.get_enum("backendModel").get_values()
+
+        if self.arch == "armv7l" and models == ["tpm-tis"]:
+            # libvirt as of 8.4.0 can advertise armv7l tpm-tis support,
+            # but then explicitly rejects that config. If we see it,
+            # assume TPM is not supported
+            # https://gitlab.com/libvirt/libvirt/-/issues/329
+            return False
+
         return len(models) > 0 and bool("emulator" in backends)
 
     def supports_graphics_spice(self):
