@@ -67,9 +67,18 @@ class _DomainDevices(XMLBuilder):
 
 class _IOThreadID(XMLBuilder):
     XML_NAME = "iothread"
-    _XML_PROP_ORDER = ["id"]
+    _XML_PROP_ORDER = ["id", "thread_pool_min", "thread_pool_max"]
 
     id = XMLProperty("./@id", is_int=True)
+    thread_pool_min = XMLProperty("./@thread_pool_min", is_int=True)
+    thread_pool_max = XMLProperty("./@thread_pool_max", is_int=True)
+
+
+class _DefaultIOThread(XMLBuilder):
+    XML_NAME = "defaultiothread"
+
+    thread_pool_min = XMLProperty("./@thread_pool_min", is_int=True)
+    thread_pool_max = XMLProperty("./@thread_pool_max", is_int=True)
 
 
 class Guest(XMLBuilder):
@@ -178,7 +187,8 @@ class Guest(XMLBuilder):
     XML_NAME = "domain"
     _XML_PROP_ORDER = [
         "type", "name", "uuid", "genid", "genid_enable",
-        "title", "description", "_metadata", "iothreads", "iothreadids",
+        "title", "description", "_metadata",
+        "iothreads", "iothreadids", "defaultiothread",
         "maxMemory", "maxMemorySlots", "memory", "_currentMemory",
         "blkiotune", "memtune", "memoryBacking",
         "_vcpus", "vcpu_current", "vcpu_placement",
@@ -224,6 +234,7 @@ class Guest(XMLBuilder):
 
     iothreads = XMLProperty("./iothreads", is_int=True)
     iothreadids = XMLChildProperty(_IOThreadID, relative_xpath="./iothreadids")
+    defaultiothread = XMLChildProperty(_DefaultIOThread)
 
     def _set_currentMemory(self, val):
         if val is not None:
@@ -618,18 +629,26 @@ class Guest(XMLBuilder):
         return True
 
     def lookup_domcaps(self):
+        def _compare_machine(domcaps):
+            capsinfo = self.lookup_capsinfo()
+            if self.os.machine == domcaps.machine:
+                return True
+            if capsinfo.is_machine_alias(self.os.machine, domcaps.machine):
+                return True
+            return False
+
         # We need to regenerate domcaps cache if any of these values change
-        def _compare(domcaps):  # pragma: no cover
+        def _compare(domcaps):
             if self.type == "test":
                 # Test driver doesn't support domcaps. We kinda fake it in
                 # some cases, but it screws up the checking here for parsed XML
                 return True
-            if self.os.machine and self.os.machine != domcaps.machine:
+            if self.os.machine and not _compare_machine(domcaps):
                 return False
             if self.type and self.type != domcaps.domain:
                 return False
             if self.os.arch and self.os.arch != domcaps.arch:
-                return False
+                return False  # pragma: no cover
             if self.emulator and self.emulator != domcaps.path:
                 return False
             return True
